//+------------------------------------------------------------------+
//|                                          Envelope ChartEvent.mq4 |
//|                        Copyright 2021, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2021, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict
#property indicator_chart_window
#property indicator_buffers 6

#property indicator_color1  clrLimeGreen
#property indicator_color2  clrBlue
#property indicator_color3  clrOrangeRed
#property indicator_color4  clrBlue
#property indicator_color5  clrBlue
#property indicator_color6  clrRed


extern int MA_Period=50;
extern double std_deviation= 0.008;

double BullishZoneBuffer[];
double BearishZoneBuffer[];
double UpperBuffer[];
double MABuffer[];
double LowerBuffer[];
double LowerBuffer2[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int OnInit()
  {
//--- indicator buffers mapping
   IndicatorDigits(Digits);
   
   SetIndexBuffer(0,BullishZoneBuffer);
   SetIndexBuffer(1,LowerBuffer); 
   SetIndexBuffer(2,BearishZoneBuffer);
   SetIndexBuffer(3,LowerBuffer2);
   SetIndexBuffer(4,UpperBuffer);
   SetIndexBuffer(5,MABuffer);

   SetIndexStyle(0,DRAW_HISTOGRAM, STYLE_DOT);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexStyle(2,DRAW_HISTOGRAM, STYLE_DOT);
   SetIndexStyle(3,DRAW_LINE);
   SetIndexStyle(4,DRAW_LINE);
   SetIndexStyle(5,DRAW_LINE);
   
   SetIndexLabel(0,NULL);
   SetIndexLabel(1,"Lower");
   SetIndexLabel(2,NULL);
   SetIndexLabel(3,"Lower");
   SetIndexLabel(4,"Upper");
   SetIndexLabel(5,"Average");

//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   int limit=rates_total-prev_calculated;
//---- main loop
   DrawLines(limit); 
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| Dynamic chart event                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,         // Event ID
                  const long& lparam,   // Parameter of type long event
                  const double& dparam, // Parameter of type double event
                  const string& sparam  // Parameter of type string events
  ){
  
  int limit;
  
  if (id==CHARTEVENT_KEYDOWN){
  Print(lparam);  
   if (lparam==68){
      
      if (std_deviation > 0){
         std_deviation -= 0.001;}
      Comment(StringFormat("Current std_deviation = %g\n", std_deviation));
      limit = iBars(NULL, PERIOD_CURRENT);
      DrawLines(limit); 
      }
      
   if (lparam==65){
   
      std_deviation += 0.001;
      Comment(StringFormat("Current std_deviation = %g\n", std_deviation));
      limit = iBars(NULL, PERIOD_CURRENT);
      DrawLines(limit); 
      }
      
   }
  }

//+------------------------------------------------------------------+
//| Draw lines                                                       |
//+------------------------------------------------------------------+
void DrawLines(int limit){

   for(int i=0; i<limit; i++)
     {
      //---- ma_shift set to 0 because SetIndexShift called abowe
      MABuffer[i]=iMA(NULL, PERIOD_CURRENT, MA_Period, 0, MODE_SMA, PRICE_CLOSE, i);
      UpperBuffer[i]= MABuffer[i] + MABuffer[i]*std_deviation;
      
      
      if (i > 100){
         BullishZoneBuffer[i] = MABuffer[i] + MABuffer[i]*std_deviation;
         BearishZoneBuffer[i] = EMPTY_VALUE;
         LowerBuffer[i]= MABuffer[i] - MABuffer[i]*std_deviation;
         LowerBuffer2[i] = EMPTY_VALUE;
         }
      else {
         BearishZoneBuffer[i] = MABuffer[i] + MABuffer[i]*std_deviation;
         BullishZoneBuffer[i] = EMPTY_VALUE;
         LowerBuffer[i] = EMPTY_VALUE;
         LowerBuffer2[i]= MABuffer[i] - MABuffer[i]*std_deviation;
         }
     }
   
   }
